from tqdm import tqdm
from typing import List, Optional, Iterable, Dict, cast
from isahitlab.actions.base import BaseAction
from isahitlab.api.task.api import TaskApi
from isahitlab.api.project_configuration.api import ProjectConfigurationApi
from isahitlab.domain.task import TaskFilters, TaskStatus, TaskOptionalFields, TaskId, TaskCompatibilityMode
from isahitlab.domain.batch import BatchId
from isahitlab.operations.project_configuration.get_project_configuration import GetProjectConfigurationOperation
from isahitlab.formatters import get_compatibility_formatter

from typeguard import typechecked

class ProjectConfigurationActions(BaseAction):
    """Project configuration actions"""

    @typechecked
    def project_configuration(
        self,
        project_id: str,
        disable_progress_bar: Optional[bool] = False
    ) -> Dict:
        """ Get the configuration of a project
        
        Args:
            project_id: ID of the project
            disable_progress_bar: Disable the progress bar display

        """

        operation = GetProjectConfigurationOperation(self.http_client)

        return operation.run(project_id=project_id, disable_progress_bar=disable_progress_bar)

    


import uuid
import logging
import json
from tqdm import tqdm
from typing import List, Optional, Iterable, Dict, Union
from isahitlab.helpers.resource import is_url, is_resource
from isahitlab.helpers.list import list_duplicates, remove_duplicates
from isahitlab.operations.base import BaseAction
from isahitlab.api.dataset.api import DatasetApi
from isahitlab.domain.dataset import FilePayload, DatasetId
from isahitlab.domain.batch import BatchId
from isahitlab.domain.project import ProjectId

from typeguard import typechecked

logger = logging.getLogger('isahitlab.append_to_dataset')


class AppendToDatasetOperation(BaseAction):
    """Upload files to a dataset

    """


    _dataset_api : DatasetApi

    @typechecked
    def run(
        self,
        project_id: ProjectId,
        dataset_id: BatchId,
        files: List[FilePayload],
        disable_progress_bar: Optional[bool] = False,
    ) -> Iterable[Dict]:
        """ Add resources to a dataset
        
            project_id: ID of the project
            dataset_id: ID of the dataset_id
                You can also provide a batch_id to use the dataset linked to the batch
            batch_id: ID of the batch
            files : list of the file to create, str or Dict -> FilePayload(path : str, file : str)
            disable_progress_bar: Disable the progress bar display
        """
        self._dataset_api = DatasetApi(self._http_client)
        nb_files = len(files)
        
        with tqdm(total=nb_files,  disable=disable_progress_bar, desc="Uploading files... ") as loader:
            for file in files:
                self._upload_file(file.file, file.path, dataset_id, project_id)
                loader.update(1)
            

        tasks = []
        
        return tasks
    
    def _upload_file(self, resource: str, folder: str, dataset_id: DatasetId, project_id: ProjectId) -> Dict:
        with open(resource, 'rb') as f:
            file_info = self._dataset_api.upload_file(dataset_id=dataset_id, project_id=project_id, file=f, folder=folder)
        return file_info




from typing import Dict
from isahitlab.api.base import BaseApi
from isahitlab.domain.batch import BatchId
from ..helpers import get_response_json, log_raise_for_status

class BatchApi(BaseApi):
    """Batch API Calls"""

    def get_batch_by_id(self, batch_id : BatchId) -> Dict :
        """Get batch"""

        project_configuration = self._http_client.get('api/task-manager/batches/{}'.format(batch_id))

        log_raise_for_status(project_configuration)

        return get_response_json(project_configuration)



"""Task domain"""
from typing import Optional
from dataclasses import dataclass



@dataclass
class PaginationFilters:
    """Pagination filters."""

    pagination: bool = True
    page: Optional[int] = 1
    limit: Optional[int] = 10
